@@ -18,7 +18,7 @@ from fastimport.commands import (
 
 from swh.model import hashutil
 from swh.model.from_disk import DentryPerms, mode_to_perms
-from swh.model.identifiers import ObjectType
+from swh.model.swhids import ObjectType
 from swh.model.toposort import toposort
 from swh.vault.cookers.base import BaseVaultCooker
 from swh.vault.cookers.utils import revision_log
